# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify Alma, CentOS, CentOS Stream and Rocky Linux as rhel
%if "%{__distinit}" == "a" || "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "cs" || "%{__distinit}" == "rl"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

Name:		perl-Tree-DAG_Node
Version:	1.35
Release:	1.%{__distinit}%{__distvers}
Summary:	Class for representing nodes in a tree
License:	GPL-1.0-or-later OR Artistic-1.0-Perl
URL:		https://metacpan.org/release/Tree-DAG_Node
Source0:	https://cpan.metacpan.org/modules/by-module/Tree/Tree-DAG_Node-%{version}.tgz
BuildArch:	noarch
# Module Build ---------------------------------------------------------------
BuildRequires:	coreutils
BuildRequires:	make
BuildRequires:	perl-generators
BuildRequires:	perl-interpreter
BuildRequires:	perl(ExtUtils::MakeMaker) >= 6.76
BuildRequires:	sed
# Module Runtime -------------------------------------------------------------
BuildRequires:	perl(File::Slurper) >= 0.014
BuildRequires:	perl(strict)
BuildRequires:	perl(warnings)
# Test Suite -----------------------------------------------------------------
BuildRequires:	findutils
BuildRequires:	perl(File::Spec) >= 3.4
BuildRequires:	perl(File::Temp) >= 0.19
BuildRequires:	perl(Test::More) >= 1.001002
BuildRequires:	perl(Test::Pod) >= 1.48
BuildRequires:	perl(utf8)
# Dependencies ---------------------------------------------------------------
%if 0%{?fedora} < 38 && 0%{?rhel} < 10
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
%endif

%description
This class encapsulates/makes/manipulates objects that represent nodes in a
tree structure. The tree structure is not an object itself, but is emergent
from the linkages you create between nodes. This class provides the methods
for making linkages that can be used to build up a tree, while preventing you
from ever making any kinds of linkages that are not allowed in a tree (such as
having a node be its own mother or ancestor, or having a node have two
mothers).

%prep
%setup -q -n Tree-DAG_Node-%{version}

# Fix up shellbangs in example scripts
sed -i -e 's|^#!/usr/bin/env perl|#!/usr/bin/perl|' scripts/*.pl

%build
perl Makefile.PL INSTALLDIRS=vendor NO_PACKLIST=1 NO_PERLLOCAL=1
%{make_build}

%install
%{make_install}
%{_fixperms} -c %{buildroot}

%check
make test
make test TEST_FILES="$(echo $(find xt/ -name '*.t'))"

%files
%license LICENSE
%doc Changes README scripts/ SECURITY.md
%{perl_vendorlib}/Tree/
%{_mandir}/man3/Tree::DAG_Node.3*

%changelog
* Mon May 12 2025 Paul Howarth <paul@city-fan.org> - 1.35-1
- Update to 1.35
  - Change sub hashref2string() so it does not generate an error or warning when
    the node or node name is '' (empty string), undef or 0 (zero)
  - Change test data so that when stringifying a tree, node names without
    attributes are output as (e.g.) 'lexeme: Attributes{}' rather than as just
    'lexeme'; this was supposed to have been done in 1.34, but was not done
    properly
  - The previous point also affects the related module Data::RenderAsTree,
    which ships with a range of test programs and sample scripts that futher
    exercise these features; the point of these latter samples is so that you
    can use them without having to fiddle about removing the test code in the
    corresponding test scripts

* Mon Apr 28 2025 Paul Howarth <paul@city-fan.org> - 1.34-1
- Update to 1.34
  - Fix test failures due to line endings on Windows (GH#4)
  - The 2 test files t/tree.with*.txt ship as ISO-8859-1, while
    t/tree.utf8.attributes.txt ships as UTF-8; so...
    - Don't explicitly use UTF-8 encoding in DAG_Node.pm's sub read_tree();
      rather, use a regexp to standardize line endings after reading, and
      likewise, in t/read.tree.t, do the same
    - Un-comment the 2 extra tests at the end of t/read.tree.t; specifically,
      line 50 was 'for (qw/utf8/)# with without/)' and now says
      'for (qw/utf8 with without/)'
    - Reformat test data files t/tree.*.attributes.txt; this was done with the
      new files share/read.write.tree.(pl, sh), and it was done because some of
      the test data files had been written by old code with slightly different
      indenting
  - Add a new test data file, t/metag.cooked.tree.txt; it's a copy of
    MarpaX::Grammar::Parser's share/metag.cooked.tree, and the test program
    t/read.tree.t was edited to include this new file
  - Patch tree::DAG_Node.format_node() to output 'Attributes: {}' and not just
    'Attributes:' when the user does not want attributes reported; this makes
    the code match the sample trees shipped in t/

* Sun Mar 30 2025 Paul Howarth <paul@city-fan.org> - 1.33-1
- Update to 1.33
  - Replace the discouraged File::Slurp::Tiny with File::Slurper
  - Add a security policy file SECURITY.md
  - Update Makefile.PL to include both ExtUtils::MakeMaker and perl
- Use %%{make_build} and %%{make_install}

* Thu Jun  1 2023 Paul Howarth <paul@city-fan.org> - 1.32-7
- Use SPDX-format license tag

* Tue Feb  2 2021 Paul Howarth <paul@city-fan.org> - 1.32-1
- Update to 1.32
  - Update Makefile.PL and POD to change RT to GitHub
- Switch upstream from search.cpan.org to metacpan.org

* Wed Feb 14 2018 Paul Howarth <paul@city-fan.org> - 1.31-1
- Update to 1.31
  - Clarify licence issue by changing the reference in the DAG_Node.pm file
    from Artistic V2 to Perl, so it now matches what I preemptively put in
    Makefile.PL and the LICENSE file; Sean Burke has kindly agreed to this
    change (GH#1)
- License changed to GPL+ or Artistic

* Fri Jan 26 2018 Paul Howarth <paul@city-fan.org> - 1.30-1
- Update to 1.30
  - Explicitly escape { and } in a regexp because unescaped { issues a warning
    now and will become a fatal error in Perl 5.32
  - Adopt new repo structure, see:
    http://savage.net.au/Ron/html/My.Workflow.for.Building.Distros.html
- BR: perl-generators unconditionally
- BR: perl-interpreter rather than perl
- Drop support for EOL distributions prior to F-13
  - Drop BuildRoot: and Group: tags
  - Drop explicit buildroot cleaning in %%install section
  - Drop explicit %%clean section
  - Drop workaround for building with Test::More < 0.88
  - Drop workaround for building with File::Temp < 0.19
  - Use upstream version requirement for File::Spec

* Fri Oct 14 2016 Paul Howarth <paul@city-fan.org> - 1.29-2
- BR: perl-generators where available
- Simplify find command using -delete

* Tue Mar  1 2016 Paul Howarth <paul@city-fan.org> - 1.29-1
- Update to 1.29
  - No code changes
  - Rework Makefile.PL so File::Spec, File::Temp and Test::More are in
    TEST_REQUIRES (CPAN RT#112568)
  - Expand the SYNOPSIS
  - Update MANIFEST.SKIP to include .gitignore
  - Test::Pod is only a developer dependency

* Fri Feb 26 2016 Paul Howarth <paul@city-fan.org> - 1.27-2
- Fix FTBFS where File::Temp is not in the minimal buildroot

* Sun Jul 12 2015 Paul Howarth <paul@city-fan.org> - 1.27-1
- Update to 1.27
  - Remove the line that reads 'use open qw(:std :utf8);' (CPAN RT#105798)
  - Remove Build.PL, ship only Makefile.PL
  - Remove .gitignore from MANIFEST

* Mon Jun 22 2015 Paul Howarth <paul@city-fan.org> - 1.26-3
- Specify all dependencies (#1234366)

* Wed Jun 17 2015 Paul Howarth <paul@city-fan.org> - 1.26-2
- Perl 5.22 rebuild

* Sun Apr 12 2015 Paul Howarth <paul@city-fan.org> - 1.26-1
- Update to 1.26
  - Fix bug in string2hashref(), which failed on some strings, such as
    {a => 'b, c'}
  - Add t/string2hash.t to test new code
- Update patch for building with old Test::More versions

* Mon Mar 23 2015 Paul Howarth <paul@city-fan.org> - 1.25-1
- Update to 1.25
  - Deleted the undocumented sub _dump_quote(), which butchered Unicode
    characters when it tried to convert ASCII control characters into printable
    strings (on the assumption all data is ASCII); methods that used to call
    _dump_quote() now just output the node's name by calling quote_name(),
    which is discussed next, and undefined names are output as the string
    'undef'
  - Add method quote_name(), which simply returns its input string surrounded
    by single-quotes
  - Add method decode_lol(), which converts the output of tree_to_lol() and
    tree_to_simple_lol() into something that is easy to read
  - Re-order a couple of methods called tree_*(), so that they are in
    alphabetical order
  - Expand the docs for methods tree_to_*(), regarding undefined node names
  - Add scripts/write.tree.pl, which creates the test input file
    t/tree.utf8.attributes.txt (note: this file is now much more complex than
    in previous versions)
  - Add scripts/read.tree.pl, and its output file scripts/read.tree.log; this
    program demonstrates the output produced by various methods
  - Fix the faulty syntax I had used in Build.PL to identify the github repo
  - Delete and re-create github repo after 'git push' failed to upload the new
    version
  - Add LICENSE file to MANIFEST
- Package LICENSE and scripts (as documentation)

* Sun Jan 25 2015 Paul Howarth <paul@city-fan.org> - 1.24-1
- Update to 1.24
  - Clean up discussion in docs of original author's reluctance to allow
    parameters to new()
  - Rewrite bareword filehandles to use a variable (my $fh)
  - Rename github repo from Tree--DAG_Node to Tree-DAG_Node - my new standard;
    update Build.PL and Makefile.PL to match
  - Reformat the docs to be ≤ 100 chars per line - my new standard
  - Change horizontal indentation used by node2string() to add 1 space, so '|'
    lines up underneath the first char of the previous node's name

* Tue Oct 21 2014 Paul Howarth <paul@city-fan.org> - 1.23-1
- Update to 1.23
  - Change output format when using node2string(), which is called by
    tree2string(); indentation that used to be '|---' is now '|--- ', which
    makes the difference between node names ''/'-', '1'/'-1', etc. much clearer
- Classify buildreqs by usage
- Update patch for building with old Test::More versions

* Thu Aug 28 2014 Paul Howarth <paul@city-fan.org> - 1.22-2
- Drop %%defattr, redundant since rpm 4.4

* Thu Feb 13 2014 Paul Howarth <paul@city-fan.org> - 1.22-1
- Update to 1.22
  - Switch from File::Slurp to File::Slurp::Tiny (CPAN RT#92976)
- Update patch for building with old Test::More versions

* Fri Jan 31 2014 Paul Howarth <paul@city-fan.org> - 1.20-1
- Update to 1.20
  - Bump File::Temp version requirement back up to 0.19, as that's the version
    that introduced the newdir() method, as used in the test suite

* Thu Jan 30 2014 Paul Howarth <paul@city-fan.org> - 1.19-1
- Update to 1.19
  - Set pre-req File::Temp version # to 0 (back from 0.2301)

* Thu Sep 19 2013 Paul Howarth <paul@city-fan.org> - 1.18-1
- Update to 1.18
  - No changes, code or otherwise, except for the version # in the *.pm, this
    file, and Changelog.ini
  - Somehow a corrupted version got uploaded to search.cpan.org, so I've just
    changed the version # (the file on MetaCPAN was fine)

* Mon Sep 16 2013 Paul Howarth <paul@city-fan.org> - 1.17-1
- Update to 1.17
  - Write test temp files in :raw mode as well as utf-8, for MS Windows users
  - Take the opportunity to change all utf8 to utf-8, as per the docs for Encode,
    except for 'use warnings qw(FATAL utf8);', which doesn't accept utf-8 :-(

* Mon Sep  9 2013 Paul Howarth <paul@city-fan.org> - 1.16-1
- Update to 1.16
  - Merge patch (slightly modified by me) from Tom Molesworth (CPAN RT#88501):
    - Remove 'use open qw(:std :utf8);' because of its global effect
    - Replace Perl6::Slurp with File::Slurp, using the latter's binmode option
      for the encoding
    - Fix docs where I'd erroneously said File::Slurp didn't support utf8

* Fri Sep  6 2013 Paul Howarth <paul@city-fan.org> - 1.15-1
- Update to 1.15
  - Replace Path::Tiny with File::Spec, because the former's list of
    dependencies is soooo long :-( (CPAN RT#88435)
  - Move t/pod.t to xt/author/pod.t
- Explicitly run the author tests
- Reinstate support for old distributions back to FC-3
- Use a locally bundled copy of File::Temp for the test suite if necessary,
  as we need the newdir method, which is only available from version 0.19

* Thu Sep  5 2013 Paul Howarth <paul@city-fan.org> - 1.14-1
- Update to 1.14
  - Document the copy() method
  - Patch the copy() method so it respects the {no_attribute_copy => 1} option
  - Add method read_tree(), for text files; it uses Perl6::Slurp, which
    supports utf8
  - Add methods read_attributes() and string2hashref($s) for use by read_tree()
  - Add t/read.tree.t to test read_tree()
  - Add t/tree.utf8.attributes.txt, in utf8, for use by t/read.tree.t
  - Add t/tree.with.attributes.txt and t/tree.without.attributes.txt for use by
    t/read.tree.t
  - Make Perl 5.8.1 a pre-req so we have access to the utf8 pragma
- Drop support for old distributions since we need Path::Tiny, which isn't
  available there

* Mon Aug 12 2013 Paul Howarth <paul@city-fan.org> - 1.13-1
- Update to 1.13
  - Change the values accepted for the no_attributes option from undef and 1
    to 0 and 1; if undef is used, it becomes 0, so pre-existing code will not
    change behaviour, whilst this makes it easier to pass 0 or 1 from the
    command line

* Tue Jul 16 2013 Paul Howarth <paul@city-fan.org> - 1.12-2
- Perl 5.18 rebuild

* Wed Jul  3 2013 Paul Howarth <paul@city-fan.org> - 1.12-1
- Update to 1.12
  - Change text in README referring to licence to match text in body of source,
    since it was in conflict with the Artistic Licence V 2.0
  - Rename CHANGES to Changes as per CPAN::Changes::SPEC
  - Various spelling fixes in the docs

* Mon Feb  4 2013 Paul Howarth <paul@city-fan.org> - 1.11-1
- Update to 1.11
  - License clarified as Artistic 2.0 (CPAN RT#83088)

* Fri Feb  1 2013 Paul Howarth <paul@city-fan.org> - 1.10-1
- Update to 1.10
  - Look for but don't require Test::Pod ≥ 1.45 (CPAN RT#83077)
- Drop redundant BR: perl(Carp)

* Fri Nov  9 2012 Paul Howarth <paul@city-fan.org> - 1.09-1
- Update to 1.09
  - No code changes
  - For pre-reqs such as strict, warnings, etc., which ship with Perl, set the
    version requirement to 0 (CPAN RT#80663)

* Fri Nov  2 2012 Paul Howarth <paul@city-fan.org> - 1.07-1
- Update to 1.07
  - New maintainer: Ron Savage
  - Pre-emptive apologies for any changes which are not back-compatible; no
    such problems are expected, but the introduction of new methods may
    disconcert some viewers
  - Fix CPAN RT#78858 and audit code for similar problems
  - Fix CPAN RT#79506
  - Rename ChangeLog to CHANGES, and add Changelog.ini
  - Replace all uses of cyclicity_fault() and Carp::croak with die
  - Remove unused methods: decommission_root(), cyclicity_allowed(),
    cyclicity_fault(), inaugurate_root(), no_cyclicity() and _update_links();
    OK - cyclicity_fault() was called once - it just died
  - Add methods: format_node(), hashref2string(), is_root(), node2string(),
    tree2string()
  - Reformat the POD big-time
  - Add Build.PL
  - Re-write Makefile.PL
  - Remove use vars(@ISA $Debug $VERSION), and replace latter 2 with 'our ...'
  - Rename t/00_about_verbose.t to t/about.perl.t
  - Add scripts/cut.and.paste.subtrees.pl (Warning: Some trees get into an
    infinite loop)
  - Add t/cut.and.paste.subtrees.t (Warning: Some trees get into an infinite
    loop)
  - Document the options (discouraged by Sean) supported in the call to
    new($hashref)
- This release by RSAVAGE -> update source URL
- BR: perl(Test::More) and perl(Test::Pod) ≥ 1.00
- Add patch to support building with Test::More < 0.88 on older distributions

* Tue Jun 12 2012 Paul Howarth <paul@city-fan.org> - 1.06-13
- BR: perl(Carp) and perl(Test)
- Don't need to remove empty directories from the buildroot
- Drop support for old distributions prior to FC-3:
  - Don't need to define %%{perl_vendorlib}
  - Use DESTDIR rather than PERL_INSTALL_ROOT

* Wed Jun 29 2011 Paul Howarth <paul@city-fan.org> - 1.06-11
- Rebuild for perl 5.14.1 in Rawhide
- Nobody else likes macros for commands
- Fix dist tag for CentOS 6 and Scientific Linux

* Thu Jun 24 2010 Paul Howarth <paul@city-fan.org> - 1.06-6
- Rebuild for perl 5.12.1 in Rawhide

* Fri May 28 2010 Paul Howarth <paul@city-fan.org> - 1.06-5
- Fix dist tag for RHEL-6 Beta

* Fri Mar  5 2010 Paul Howarth <paul@city-fan.org> - 1.06-4
- Dist tag for Rawhide no longer needs special-casing
- Use %%{_fixperms} macro instead of our own %%{__chmod} incantation

* Thu Oct  1 2009 Paul Howarth <paul@city-fan.org> - 1.06-3
- Define RPM macros in global scope

* Fri Nov 14 2008 Paul Howarth <paul@city-fan.org> - 1.06-2
- Tweak dist tag macros to work on current Rawhide with three-part releasenum

* Mon Dec 10 2007 Paul Howarth <paul@city-fan.org> - 1.06-1
- Update to 1.06
- Clarify license as GPL version 1 or later, or Artistic (i.e. same as perl)
- Update source URL for new upstream maintainer

* Mon Mar 19 2007 Paul Howarth <paul@city-fan.org> - 1.05-4
- Fix dist tag for development builds and Fedora 7 onwards
- Remove %%{__perl_version} macro, used only once
- Define %%{__id_u} in a more portable way
- Use search.cpan.org download URL
- BuildRequire perl(ExtUtils::MakeMaker)
- Remove buildroot unconditionally in %%clean and %%install
- Don't use macros in command paths, hardcode them instead
- Fix argument order for find with -depth

* Fri Aug 26 2005 Paul Howarth <paul@city-fan.org> - 1.05-3
- Import from Fedora Extras

* Fri Jul  1 2005 Jose Pedro Oliveira <jpo at di.uminho.pt> - 1.05-2
- Dist tag.

* Thu Dec 30 2004 Jose Pedro Oliveira <jpo at di.uminho.pt> - 0:1.05-0.fdr.1
- Update to 1.05.

* Sun Jul 04 2004 Jose Pedro Oliveira <jpo at di.uminho.pt> - 0:1.04-0.fdr.1
- First build.
